# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Tests runner."""

# Standard library imports
import os
import sys

# Third party imports
import pytest
import qtpy

qtpy

# Constants
HERE = os.path.dirname(os.path.abspath(__file__))
MAC = sys.platform == 'darwin'
WIN = os.name == 'nt'
LINUX = sys.platform.startswith('linux')
PYTEST_XDIST = True

PY2 = sys.version_info.major == 2
PY3 = sys.version_info.major == 3

try:
    import xdist
    xdist
except ImportError:
    PYTEST_XDIST = False


def update_covarage_rc():
    """Add no cover lines based on operating system."""
    coverage_path = os.path.join(HERE, '.coveragerc')
    coverage_rc_content = """# This file is autogenerated by runscripts.py
# Do not make changes here!!!
[run]
omit =
    */tests/*
    */external/*
    */app/Navigator.app/*
[report]
fail_under = 0
show_missing = true
skip_covered = true
exclude_lines =
    pragma: no cover
    def local_test():
    if __name__ == .__main__.:"""

    updated_content = coverage_rc_content
    if WIN:
        updated_content += '\n    pragma: no cover unix'
        updated_content += '\n    pragma: no cover mac'
        updated_content += '\n    pragma: no cover linux'
    elif MAC:
        updated_content += '\n    pragma: no cover win'
        updated_content += '\n    pragma: no cover linux'
    elif LINUX:
        updated_content += '\n    pragma: no cover win'
        updated_content += '\n    pragma: no cover mac'

    if PY2:
        updated_content += '\n    pragma: no cover py3'
    elif PY3:
        updated_content += '\n    pragma: no cover py2'

    print('\nUpdating .coveragerc with:')
    print('--------------------------')
    print(updated_content)
    with open(coverage_path, 'w') as f:
        f.write(updated_content)


update_covarage_rc()

if 'pre-commit' in sys.argv:
    pytest_args = [
        'anaconda_navigator', '-q', '--disable-warnings', '--cov-report', 'term',
        '--cov=anaconda_navigator', '--cov-fail-under=54',
    ]
else:
    pytest_args = [
        '-x',
        'anaconda_navigator',
        '-rw',
        '--durations=10',
        '--cov=anaconda_navigator',
        '--cov-report',
        'term-missing',
    ]

if PYTEST_XDIST:
    pytest_args.extend(['-n', '1'])
print('\nPytest arguments:')
print('-----------------')
print(pytest_args)
print('\n\n')
sys.exit(pytest.main(pytest_args))



from qtpy.QtCore import Qt
from unittest import mock


class TestBaseLoginPage:
    def test_dialog_clicked_back_buttons(self):
        self.qtbot.wait_for_window_shown(self.widget)

        with self.qtbot.wait_signal(self.widget.button_back.clicked,
                                    raising=True), mock.patch.object(self.widget, 'reject'):
            self.qtbot.mouseClick(self.widget.button_back, Qt.LeftButton)
            self.widget.reject.assert_called()
            self.widget.main_dialog.show.assert_called()

    def test_login_button_enabled(self):
        self.qtbot.wait_for_window_shown(self.widget)

        self.widget.text_username.setText("TEST_USER")
        self.widget.text_password.setText("TEST_PASSWORD")
        self.widget.text_username.textEdited.emit(None)
        assert self.widget.button_login.isEnabled() is True

        self.widget.text_username.setText("")
        self.widget.text_password.setText("")
        self.widget.text_username.textEdited.emit(None)
        assert self.widget.button_login.isEnabled() is False

    def test_login(self):
        self.qtbot.wait_for_window_shown(self.widget)

        self.widget.text_username.setText("TEST_USER")
        self.widget.text_password.setText("TEST_PASSWORD")
        self.widget.text_username.textEdited.emit(None)

        with mock.patch.object(self.widget.config, 'get'), \
                self.qtbot.wait_signal(self.widget.button_login.clicked, raising=True):

            self.widget.config.get.return_value = "https://api.domain.org"
            self.widget.anaconda_api.login.return_value = mock.Mock()
            self.qtbot.wait(1500)
            self.qtbot.mouseClick(self.widget.button_login, Qt.LeftButton)
            self.widget.anaconda_api.client_reload.assert_called()

    def test_login_finished(self):
        worker, output, error = mock.Mock(), "TOKEN_MOCK", ""
        self.widget._finished(worker, output, error)
        with mock.patch.object(self.widget, 'accept'):
            self.widget.accept.assset_called()
            self.widget.main_dialog.accept.assert_called()

    def test_login_finished_with_error(self):
        worker, output, error = mock.Mock(), "", "error"
        with self.qtbot.wait_signal(self.widget.sig_authentication_failed, raising=True):
            self.widget._finished(worker, output, error)
        with mock.patch.object(self.widget.tracker, 'track_event'), mock.patch.object(self.widget, 'accept'):
            self.widget.accept.assset_not_called()
            self.widget.main_dialog.accept.assset_not_called()
            self.widget.tracker.track_event.accept.assset_called()


class TestBaseSetDomainPage:
    def test_set_domain(self):
        domain = "https://api.someedition.com"
        self.widget.input_line.setText(domain)

        with mock.patch.object(self.widget, 'check_text'), mock.patch.object(self.widget, 'accept'), \
                mock.patch.object(self.widget, 'anaconda_api'), mock.patch.object(self.widget.config, 'set'), \
                self.qtbot.wait_signal(self.widget.button_apply.clicked, raising=True):

            self.widget.check_text.return_value = True
            self.qtbot.mouseClick(self.widget.button_apply, Qt.LeftButton)
            self.widget.accept.assert_called()
            self.widget.check_text.assert_called()
            self.widget.anaconda_api.client_reload.assert_called()
            self.widget.config.set.assert_called()

    def test_set_domain_failed(self):
        with mock.patch.object(self.widget, 'check_text'), mock.patch.object(self.widget, 'accept'), \
                mock.patch.object(self.widget, 'anaconda_api'), mock.patch.object(self.widget.config, 'set'), \
                self.qtbot.wait_signal(self.widget.button_apply.clicked, raising=True):
            self.widget.check_text.return_value = False
            self.qtbot.mouseClick(self.widget.button_apply, Qt.LeftButton)
            self.widget.check_text.assert_called()
            self.widget.accept.assert_not_called()
            self.widget.anaconda_api.client_reload.assert_not_called()
            self.widget.config.set.assert_not_called()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Bootstrapping script."""

# Standard library imports
import optparse
import os
import os.path as osp
import sys

# Local imports
from anaconda_navigator.app import main

# =============================================================================
#  Parse command line
# =============================================================================
parser = optparse.OptionParser(
    usage="python bootstrap.py [options] [-- anaconda_navigator_options]",
    epilog="""\
Arguments for main script are specified after the --
symbol (example: `python bootstrap.py -- --foo-bar`).
"""
)
parser.add_option(
    '--gui',
    default=None,
    help="GUI toolkit: pyqt5 (for PyQt5) or pyqt (for PyQt4)"
)

options, args = parser.parse_args()

assert options.gui in (None, 'pyqt5', 'pyqt'), \
       "Invalid GUI toolkit option '{0}'".format(options.gui)

# Prepare arguments for main script
sys.argv = [sys.argv[0]] + args

# =============================================================================
#  Start execution
# =============================================================================
# Get current dir
print("Executing from source checkout")
DEVPATH = osp.dirname(osp.abspath(__file__))

# Warn if we are is located on non-ASCII path
try:
    osp.join(DEVPATH, 'test')
except UnicodeDecodeError:
    print(
        "STOP: Anaconda Navigator is located in the path with non-ASCII "
        "characters,"
    )
    print("      which is known to cause problems.")
    try:
        input = raw_input
    except NameError:
        pass
    input("Press Enter to continue or Ctrl-C to abort...")

# Patch sys.path
sys.path.insert(0, DEVPATH)
print("01. Added {0} to sys.path".format(DEVPATH))

# =============================================================================
#  Selecting the GUI toolkit
# =============================================================================
if options.gui is None:
    try:
        import PyQt5  # analysis:ignore
        print("02. PyQt5 is detected, selecting")
        os.environ['QT_API'] = 'pyqt5'
    except ImportError:
        try:
            import PyQt4  # analysis:ignore
            print("02. PyQt4 is detected, selecting")
            os.environ['QT_API'] = 'pyqt'
        except ImportError:
            print("02. No PyQt5 or PyQt4 detected, please install one of them")
else:
    print("02. Skipping GUI toolkit detection")
    os.environ['QT_API'] = options.gui

# =============================================================================
#  Executing Anaconda Navigator
# =============================================================================
print("04. Executing the application")

main.main()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Tests for valid urls in bundled content."""

# yapf: disable

# Standard library imports
import io
import json

# Third party imports
import pytest
import requests

# Local imports
from anaconda_navigator.static import content


# yapf: enable


# --- Tests
# -----------------------------------------------------------------------------
@pytest.mark.skip
def test_content_urls():
    with open(content.LINKS_INFO_PATH, 'r') as f_handle:
        raw_data = f_handle.read()
    data = json.loads(raw_data)

    for item in data:
        uri = item.get('uri')

        try:
            r = requests.head(uri, timeout=10, verify=False)
            code = r.status_code
            error = None
        except Exception as e:
            code = None
            error = e

        # Temporal bypass for planet.scipy which is failing
        if 'planet.scipy' not in uri:
            print(uri, code, error)
            assert code in [200, 301, 302, 403]


def test_content_encoding():
    """Make sure the bundled content is ascii."""
    # See: https://github.com/ContinuumIO/navigator/issues/1613
    with io.open(content.LINKS_INFO_PATH, 'r', encoding='ascii') as f_handle:
        f_handle.read()



import pytest
from unittest import mock

from anaconda_navigator.widgets.dialogs.login import (
    MainLoginDialog, AnacondaLoginPage, EnterpriseRepoLoginPage, EnterpriseRepoSetDomainPage,
    CommercialEditionLoginPage, TeamEditionLoginPage, TeamEditionAddChannelsPage, TeamEditionSetDomainPage, QWidget
)


def _get_main_dialog_mock():
    main_mock = mock.Mock()
    main_mock.parent = QWidget()
    return main_mock


def dialog_bot(request, qtbot, widget):
    request.cls.qtbot = qtbot
    request.cls.widget = widget

    qtbot.addWidget(widget)
    widget.show()


@pytest.fixture
def main_dialog_bot(request, qtbot):
    dialog_bot(request, qtbot, MainLoginDialog())


@pytest.fixture
def anaconda_org_dialog_bot(request, qtbot):
    dialog_bot(request, qtbot, AnacondaLoginPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def commercial_dialog_bot(request, qtbot):
    return dialog_bot(request, qtbot, CommercialEditionLoginPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def enterprise_domain_set_dialog_bot(request, qtbot):
    return dialog_bot(request, qtbot, EnterpriseRepoSetDomainPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def enterprise_login_page_dialog_bot(request, qtbot):
    request.cls.config_url_option = "anaconda_api_url"
    return dialog_bot(request, qtbot, EnterpriseRepoLoginPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def team_domain_set_dialog_bot(request, qtbot):
    return dialog_bot(request, qtbot, TeamEditionSetDomainPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def anaconda_org_dialog_bot(request, qtbot):
    dialog_bot(request, qtbot, AnacondaLoginPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def commercial_dialog_bot(request, qtbot):
    return dialog_bot(request, qtbot, CommercialEditionLoginPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def enterprise_domain_set_dialog_bot(request, qtbot):
    return dialog_bot(request, qtbot, EnterpriseRepoSetDomainPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def enterprise_login_page_dialog_bot(request, qtbot):
    request.cls.config_url_option = "anaconda_api_url"
    return dialog_bot(request, qtbot, EnterpriseRepoLoginPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def team_domain_set_dialog_bot(request, qtbot):
    return dialog_bot(request, qtbot, TeamEditionSetDomainPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def team_login_page_dialog_bot(request, qtbot):
    return dialog_bot(request, qtbot, TeamEditionLoginPage(mock.Mock(), _get_main_dialog_mock()))


@pytest.fixture
def team_add_channels_page_dialog_bot(request, qtbot):
    anaconda_api_mock = mock.Mock()
    anaconda_api_mock.get_channels.return_value = tuple()
    return dialog_bot(request, qtbot, TeamEditionAddChannelsPage(anaconda_api_mock, _get_main_dialog_mock()))



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Anaconda Navigator setup script."""

# Third party imports
from setuptools import find_packages, setup
from setuptools.command.install import install

# Local imports
from anaconda_navigator import __version__

packages = find_packages()
setup(
    name='anaconda-navigator',
    version=__version__,
    description='Anaconda Navigator',
    author='Continuum Analytics',
    packages=packages,
    package_data={
        'anaconda_navigator.static.content': ['*.json', '*.json.bz2', '*.bz2'],
        'anaconda_navigator.static.css': ['*.css', '*.scss'],
        'anaconda_navigator.static.fonts': ['*.ttf'],
        'anaconda_navigator.static.images':
        ['*.jpg', '*.jpeg', '*.png', '*.gif', '*.svg', 'icons/*'],
    },
    entry_points={
        'gui_scripts':
        ['anaconda-navigator = anaconda_navigator.app.main:main']
    },
    include_package_data=True,
)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Tests for bundled image content."""

# yapf: disable

# Standard library imports
import os

# Local imports
from anaconda_navigator.static import images


# yapf: enable


# --- Tests
# -----------------------------------------------------------------------------
def test_bundled_images_exist():
    variables = [var for var in dir(images) if var.isupper()]
    paths = [getattr(images, var) for var in variables]
    for path in paths:
        print(path)
        assert os.path.isdir(path) or os.path.isfile(path)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Test launch applications utilities."""

# Standard library imports
import os

# Third party imports
import pytest

# Local imports
from anaconda_navigator.api.conda_api import _CondaAPI
from anaconda_navigator.config import WIN
from anaconda_navigator.utils import launch
from anaconda_navigator.utils.fixtures import tmpfolder


# --- Fixtures
# -----------------------------------------------------------------------------
@pytest.fixture(scope="module")
def conda_api():
    api = _CondaAPI()
    return api


# --- Tests
# -----------------------------------------------------------------------------
@pytest.mark.skipif(WIN, reason="Skip on windows as used on unix only")
def test_is_program_installed():
    assert launch.is_program_installed('python')
    assert not launch.is_program_installed('bloop')
    assert not launch.is_program_installed('bleep')


@pytest.mark.skipif(not WIN, reason="Skip on unix as used on win only")
def test_get_command_on_win_spaces(tmpfolder):
    tmp_folder = os.path.join(
        tmpfolder,
        'dieter oms',
    )
    root_prefix = 'C:\\prefix with spaces'
    prefix = root_prefix + '\\envs\\someenv'
    command = 'boo'
    package_name = 'foobar'
    environment = None
    popen_dict = launch.get_command_on_win(
        prefix, command, package_name, root_prefix, environment=environment, default_scripts_path=tmp_folder
    )
    cmd = popen_dict.pop('cmd')
    print(popen_dict)

    assert 'call "C:\\prefix with spaces\\Scripts\\activate" "C:\\prefix with spaces\\envs\\someenv"' in cmd
    assert '"' + tmp_folder in cmd


@pytest.mark.skipif(not WIN, reason="Skip on unix as used on win only")
def test_get_command_on_win_no_spaces(tmpfolder):
    tmp_folder = os.path.join(
        tmpfolder,
        'dieter oms',
    )
    root_prefix = 'C:\\prefix-with-no-spaces'
    prefix = root_prefix + '\\envs\\someenv'
    command = 'boo'
    package_name = 'foobar'
    environment = None
    popen_dict = launch.get_command_on_win(
        prefix, command, package_name, root_prefix, environment=environment, default_scripts_path=tmp_folder
    )
    cmd = popen_dict.pop('cmd')
    print(popen_dict)

    assert 'call C:\\prefix-with-no-spaces\\Scripts\\activate C:\\prefix-with-no-spaces\\envs\\someenv' in cmd
    assert '"' + tmp_folder in cmd


@pytest.mark.skipif(not WIN, reason="Skip on unix as used on win only")
def test_get_command_on_win_no_spaces_2(tmpfolder):
    tmp_folder = os.path.join(
        tmpfolder,
        'dieter-oms',
    )
    root_prefix = 'C:\\prefix-with-no-spaces'
    prefix = root_prefix + '\\envs\\someenv'
    command = 'boo'
    package_name = 'foobar'
    environment = None
    popen_dict = launch.get_command_on_win(
        prefix, command, package_name, root_prefix, environment=environment, default_scripts_path=tmp_folder
    )
    cmd = popen_dict.pop('cmd')
    print(popen_dict)

    assert 'call C:\\prefix-with-no-spaces\\Scripts\\activate C:\\prefix-with-no-spaces\\envs\\someenv' in cmd
    assert '"' + tmp_folder not in cmd


@pytest.mark.skipif(WIN, reason="Skip on windows as used on unix only")
def test_get_command_on_unix_spaces(tmpfolder):
    tmp_folder = os.path.join(
        tmpfolder,
        'dieter oms',
    )
    root_prefix = '/prefix with spaces'
    prefix = root_prefix + '/envs/someenv'
    command = 'boo'
    package_name = 'foobar'
    environment = None
    popen_dict = launch.get_command_on_unix(
        prefix, command, package_name, root_prefix, environment=environment, default_scripts_path=tmp_folder
    )
    print(popen_dict)
    cmd = popen_dict.pop('cmd')

    assert '. "/prefix with spaces/bin/activate" "/prefix with spaces/envs/someenv"' in cmd
    assert '"' + tmp_folder in cmd


@pytest.mark.skipif(WIN, reason="Skip on windows as used on unix only")
def test_get_command_on_unix_no_spaces(tmpfolder):
    tmp_folder = os.path.join(
        tmpfolder,
        'dieter oms',
    )
    root_prefix = '/prefix-with-no-spaces'
    prefix = root_prefix + '/envs/someenv'
    command = 'boo'
    package_name = 'foobar'
    environment = None
    popen_dict = launch.get_command_on_unix(
        prefix, command, package_name, root_prefix, environment=environment, default_scripts_path=tmp_folder
    )
    print(popen_dict)
    cmd = popen_dict.pop('cmd')

    assert '. /prefix-with-no-spaces/bin/activate /prefix-with-no-spaces/envs/someenv' in cmd
    assert '"' + tmp_folder in cmd


@pytest.mark.skipif(WIN, reason="Skip on windows as used on unix only")
def test_get_command_on_unix_no_spaces_2(tmpfolder):
    tmp_folder = os.path.join(
        tmpfolder,
        'dieter-oms',
    )
    root_prefix = '/prefix-with-no-spaces'
    prefix = root_prefix + '/envs/someenv'
    command = 'boo'
    package_name = 'foobar'
    environment = None
    popen_dict = launch.get_command_on_unix(
        prefix, command, package_name, root_prefix, environment=environment, default_scripts_path=tmp_folder
    )
    print(popen_dict)
    cmd = popen_dict.pop('cmd')

    assert '. /prefix-with-no-spaces/bin/activate /prefix-with-no-spaces/envs/someenv' in cmd
    assert '"' + tmp_folder not in cmd


@pytest.mark.xfail
def test_create_app_run_script():
    # command, package_name, prefix, root_prefix, suffix
    raise NotImplementedError


@pytest.mark.xfail
def test_launch():
    # prefix, command, leave_path_alone, working_directory=HOME_PATH,
    # package_name=None, root_prefix=None
    raise NotImplementedError


@pytest.mark.xfail
def test_console(activate=None):
    raise NotImplementedError


@pytest.mark.xfail
def test_check_prog(api):
    # prog, prefix
    # 'notebook', 'jupyter notebook'
    raise NotImplementedError


@pytest.mark.xfail
def test_py_in_console():
    # activate=None, prog='python'
    raise NotImplementedError


@pytest.mark.xfail
def test_run_ex():
    # *args
    raise NotImplementedError


@pytest.mark.xfail
def test_run_python_console():
    # project_path, project=None, filename=""
    raise NotImplementedError



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2021 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Tests for login dialog."""
from qtpy.QtCore import Qt  # analysis:ignore
import pytest
from unittest import mock, TestCase
import pytestqt.qtbot as qtbot  # analysis:ignore

import anaconda_navigator.widgets.dialogs.tests.login.mixins as mixins
skip = pytest.mark.skip


@pytest.mark.usefixtures("main_dialog_bot")
class TestMainLoginDialog(TestCase):
    def test_main_dialog_clicked_login_buttons(self):
        self.qtbot.wait_for_window_shown(self.widget)
        for login_card in (
            self.widget.anaconda_client_login_card, self.widget.commercial_edition_login_card,
            self.widget.enterprise_edition_login_card
        ):

            login_page_mock = mock.Mock()
            login_card.login_page = login_page_mock
            with self.qtbot.wait_signal(self.widget.sig_login_type_clicked, raising=True):
                self.qtbot.mouseClick(login_card.button_login, Qt.LeftButton)


@pytest.mark.usefixtures("commercial_dialog_bot")
class TestCommercialLogin(TestCase):
    @mock.patch('anaconda_navigator.widgets.dialogs.login.token_set')
    def test_set_token(self, token_set_mock):
        self.widget.input_line.setText("TOKEN_MOCK")

        with mock.patch.object(self.widget, 'check_text'), mock.patch.object(self.widget, 'accept'), \
                mock.patch.object(self.widget, 'anaconda_api'), mock.patch.object(self.widget.config, 'set'), \
                self.qtbot.wait_signal(self.widget.button_apply.clicked, raising=True):

            self.widget.check_text.return_value = True
            self.qtbot.mouseClick(self.widget.button_apply, Qt.LeftButton)

            token_set_mock.assert_called_with("TOKEN_MOCK")
            self.widget.accept.assert_called()
            self.widget.check_text.assert_called()
            self.widget.anaconda_api.client_reload.assert_called()
            self.widget.config.set.assert_called()

    @mock.patch('anaconda_navigator.widgets.dialogs.login.token_set')
    def test_set_token_failed(self, token_set_mock):
        with mock.patch.object(self.widget, 'check_text'), mock.patch.object(self.widget, 'accept'), \
                mock.patch.object(self.widget, 'anaconda_api'), mock.patch.object(self.widget.config, 'set'), \
                self.qtbot.wait_signal(self.widget.button_apply.clicked, raising=True):
            self.widget.check_text.return_value = False
            self.qtbot.mouseClick(self.widget.button_apply, Qt.LeftButton)

            self.widget.check_text.assert_called()
            token_set_mock.assert_not_called()
            self.widget.accept.assert_not_called()
            self.widget.anaconda_api.client_reload.assert_not_called()
            self.widget.config.set.assert_not_called()

    @mock.patch('anaconda_navigator.widgets.dialogs.login.validate_token')
    def test_check_text(self, validate_token_mock):
        self.widget.input_line.setText("TOKEN_MOCK")
        self.assertTrue(self.widget.check_text())
        validate_token_mock.assert_called()


@pytest.mark.usefixtures("anaconda_org_dialog_bot")
class TestAnacondaOrgLogin(TestCase, mixins.TestBaseLoginPage):
    pass


@pytest.mark.usefixtures("enterprise_login_page_dialog_bot")
class TestEnterpriseLoginPage(mixins.TestBaseLoginPage, TestCase):
    pass


@pytest.mark.usefixtures("enterprise_domain_set_dialog_bot")
class TestEnterpriseSetDomainPage(mixins.TestBaseSetDomainPage, TestCase):
    pass


@pytest.mark.usefixtures("team_domain_set_dialog_bot")
class TestTeamSetDomainPage(mixins.TestBaseSetDomainPage, TestCase):
    pass


@pytest.mark.usefixtures("team_login_page_dialog_bot")
class TestTeamLoginPage(mixins.TestBaseLoginPage, TestCase):
    @mock.patch('anaconda_navigator.widgets.dialogs.login.TeamEditionAddChannelsPage')
    def test_login_finished(self, channels_page_mock):
        worker, output, error = mock.Mock(), '["token", "refresh_token"]', ""

        with mock.patch.object(self.widget, 'accept'), mock.patch.object(self.widget, 'check_text'):
            self.widget._finished(worker, output, error)
            self.widget.accept.assert_called()
            self.widget.check_text.assert_called()
            channels_page_mock.assert_called()

    def test_login_finished_with_error(self):
        worker, output, error = mock.Mock(), '{"token": "Token", "message": "Unauthorized"}', ""

        with mock.patch.object(self.widget, '_track_error'), mock.patch.object(self.widget, 'check_text'):
            self.widget._finished(worker, output, error)
            self.assertEqual(self.widget.label_message.text(), '<i>Invalid Credentials!</i>')
            self.widget._track_error.assert_called()
            self.widget.check_text.assert_called()

    def test_login_finished_with_exception(self):
        worker, output, error = mock.Mock(), '{BROKEN]', ""

        with mock.patch.object(self.widget, '_track_error'), mock.patch.object(self.widget, 'check_text'):
            self.widget._finished(worker, output, error)
            self.assertEqual(self.widget.label_message.text(), '<i>Unhandled error happened!</i>')
            self.widget._track_error.assert_called()
            self.widget.check_text.assert_called()


@skip
@pytest.mark.usefixtures("team_add_channels_page_dialog_bot")
class TestTeamAddChannelsPage(TestCase):
    def test_skip(self):
        with self.qtbot.wait_signal(self.widget.button_skip.clicked,
                                    raising=True), mock.patch.object(self.widget.anaconda_api, 'create_login_data'
                                                                     ), mock.patch.object(self.widget, 'close_window'):
            self.qtbot.mouseClick(self.widget.button_skip, Qt.LeftButton)
            self.widget.close_window.assert_called()
            self.widget.anaconda_api.create_login_data.assert_called()

    def test_add_channels(self):
        with self.qtbot.wait_signal(self.widget.button_add.clicked, raising=True), mock.patch.object(
                self.widget.anaconda_api, 'create_login_data'), mock.patch.object(self.widget, 'close_window'), \
                mock.patch.object(self.widget.channels_table, 'get_selected_channels'):

            self.widget.channels_table.get_selected_channels.return_value = (['default_channel'], ['channel'])
            self.qtbot.mouseClick(self.widget.button_add, Qt.LeftButton)
            self.widget.close_window.assert_called()
            self.widget.anaconda_api.create_login_data.assert_called()

    def test_add_channels_fallback(self):
        with self.qtbot.wait_signal(self.widget.button_add.clicked, raising=True), mock.patch.object(
                self.widget.anaconda_api, 'create_login_data'), mock.patch.object(self.widget, 'close_window'), \
                mock.patch.object(self.widget.channels_table, 'get_selected_channels'):

            self.widget.channels_table.get_selected_channels.return_value = ([], ['channel'])
            self.qtbot.mouseClick(self.widget.button_add, Qt.LeftButton)
            self.widget.close_window.assert_not_called()
            self.widget.anaconda_api.create_login_data.assert_not_called()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""
This module provides user configuration file management features.

It's based on the ConfigParser module (present in the standard library).
"""

# yapf: disable

from __future__ import print_function

# Standard library imports
import ast
import io
import os
import os.path as osp
import re
import shutil
import sys
import time

# Local imports
from anaconda_navigator.config.base import get_home_dir
from anaconda_navigator.utils.py3compat import configparser as cp
from anaconda_navigator.utils.py3compat import is_text_string, to_text_string


# yapf: enable

PY2 = sys.version[0] == '2'

# Standard imports for Python 2
if PY2:
    import codecs


# -----------------------------------------------------------------------------
# --- Auxiliary classes
# -----------------------------------------------------------------------------
class NoDefault:
    """NoDefault object."""

    pass


# -----------------------------------------------------------------------------
# --- Defaults class
# -----------------------------------------------------------------------------
class DefaultsConfig(cp.ConfigParser):
    """Class used to save defaults to a file."""
    def __init__(self, name, subfolder):
        """
        Class used to save defaults to a file.

        Parameters
        -----------
        name: str
            Name of the configuration.
        subfolder: str
            Path to folder location of configuration file.
        """
        cp.ConfigParser.__init__(self)
        self.name = name
        self.subfolder = subfolder

    def _write(self, fp):
        """Private write method for Python 2.

        The one from configparser fails for non-ascii Windows accounts.
        """
        if self._defaults:
            fp.write("[{section}]\n".format(section=cp.DEFAULTSECT))
            for (key, value) in self._defaults.items():
                write_value = str(value).replace('\n', '\n\t')
                fp.write("{key} = {value}\n".format(key=key, value=write_value))
            fp.write("\n")
        for section in self._sections:
            fp.write("[{section}]\n".format(section=section))
            for (key, value) in self._sections[section].items():
                if key == "__name__":
                    continue
                if (value is not None) or (self._optcre == self.OPTCRE):
                    value = to_text_string(value)
                    key = " = ".join((key, value.replace('\n', '\n\t')))
                fp.write("{0}\n".format(key))
            fp.write("\n")

    def _set(self, section, option, value, verbose):
        """Private set method."""
        if not self.has_section(section):
            self.add_section(section)
        if not is_text_string(value):
            value = repr(value)
        if verbose:
            print('{0}[ {1} ] = {2}'.format(section, option, value))
        cp.ConfigParser.set(self, section, option, value)

    def _save(self):
        """Save config into the associated .ini file."""
        fname = self.filename()

        def _write_file(fname):
            if PY2:
                with codecs.open(fname, 'w', encoding='utf-8') as configfile:
                    self._write(configfile)
            else:
                with open(fname, 'w', encoding='utf-8') as configfile:
                    self.write(configfile)

        try:  # the "easy" way
            _write_file(fname)
        except IOError:
            try:  # the "delete and sleep" way
                if osp.isfile(fname):
                    os.remove(fname)
                time.sleep(0.05)
                _write_file(fname)
            except Exception as e:
                print("Failed to write user configuration file.")
                print("Please submit a bug report.")
                raise (e)

    def filename(self):
        """Create a .ini filename located in user home directory."""
        folder = osp.join(get_home_dir(), self.subfolder)
        # Save defaults in a "defaults" dir of .anaconda_navigator to not
        # pollute it
        if 'defaults' in self.name:
            folder = osp.join(folder, 'defaults')
        try:
            os.makedirs(folder)
        except os.error:
            # Folder (or one of its parents) already exists
            pass
        ini_file = osp.join(folder, '{0}.ini'.format(self.name))
        return ini_file

    def set_defaults(self, defaults):
        """Set default configuration."""
        for section, options in defaults:
            for option in options:
                new_value = options[option]
                self._set(section, option, new_value, False)


# -----------------------------------------------------------------------------
# --- User config class
# -----------------------------------------------------------------------------
class UserConfig(DefaultsConfig):
    """
    User configuration class.

    Note that 'get' and 'set' arguments number and type differ from the
    overriden methods.
    """

    DEFAULT_SECTION_NAME = 'main'

    def __init__(
        self,
        name,
        defaults=None,
        load=True,
        version=None,
        subfolder=None,
        backup=False,
        raw_mode=False,
        remove_obsolete=False
    ):
        """
        User configuration class.

        Parameters
        ----------
        name: str
            Name of the configuration.
        defaults: dict
            dictionnary containing options *or* list of tuples (section_name,
            options).
        version: str
            Version of the configuration file (X.Y.Z format).
        subfolder: str
            Configuration file will be saved in HOME/subfolder/name.ini.
        """
        DefaultsConfig.__init__(self, name, subfolder)

        self.raw = 1 if raw_mode else 0
        if (version is not None and re.match('^(\d+).(\d+).(\d+)$', version) is None):
            raise ValueError("Version number {0} is incorrect - must be in " "X.Y.Z format".format(version))

        if isinstance(defaults, dict):
            defaults = [(self.DEFAULT_SECTION_NAME, defaults)]
        self.defaults = defaults
        if defaults is not None:
            self.reset_to_defaults(save=False)

        fname = self.filename()
        if backup:
            try:
                shutil.copyfile(fname, "{0}.bak".format(fname))
            except IOError:
                pass

        if load:
            # If config file already exists, it overrides Default options:
            self.load_from_ini()
            old_ver = self.get_version(version)

            # Save new defaults
            self.__save_new_defaults(defaults, version, subfolder)

            # Updating defaults only if major/minor version is different
            if self._minor(version) != self._minor(old_ver):
                if backup:
                    try:
                        shutil.copyfile(fname, "{0}-{1}.bak".format(fname, old_ver))
                    except IOError:
                        pass
                self.__update_defaults(defaults, old_ver)

                # Remove deprecated options if major version has changed
                if (remove_obsolete or self._major(version) != self._major(old_ver)):
                    self.__remove_deprecated_options(old_ver)

                # Set new version number
                self.set_version(version, save=False)
            if defaults is None:
                # If no defaults are defined, set .ini file settings as default
                self.set_as_defaults()

    @staticmethod
    def _major(_t):
        """Return major component in config versions."""
        return _t[:_t.find('.')]

    @staticmethod
    def _minor(_t):
        """Return minor component in config versions."""
        return _t[:_t.rfind('.')]

    def get_version(self, version='0.0.0'):
        """Return configuration (not application!) version."""
        return self.get(self.DEFAULT_SECTION_NAME, 'version', version)

    def set_version(self, version='0.0.0', save=True):
        """Set configuration (not application!) version."""
        self.set(self.DEFAULT_SECTION_NAME, 'version', version, save=save)

    def load_from_ini(self):
        """Load config from the associated .ini file."""
        try:
            if PY2:
                fname = self.filename()
                if osp.isfile(fname):
                    try:
                        with codecs.open(fname, encoding='utf-8') as conf_file:
                            self.readfp(conf_file)
                    except IOError:
                        print("Failed reading file", fname)
            else:
                self.read(self.filename(), encoding='utf-8')
        except cp.MissingSectionHeaderError:
            print("Warning: File contains no section headers.")

    def __load_old_defaults(self, old_version):
        """Read old defaults."""
        old_defaults = cp.ConfigParser()
        path = osp.dirname(self.filename())
        path = osp.join(path, 'defaults')
        old_defaults.read(osp.join(path, 'defaults-' + old_version + '.ini'))
        return old_defaults

    @staticmethod
    def __save_new_defaults(defaults, new_version, subfolder):
        """Save new defaults."""
        new_defaults = DefaultsConfig(
            name='defaults-' + new_version,
            subfolder=subfolder,
        )
        if not osp.isfile(new_defaults.filename()):
            new_defaults.set_defaults(defaults)
            new_defaults._save()

    def __update_defaults(self, defaults, old_version, verbose=False):
        """Update defaults after a change in version."""
        old_defaults = self.__load_old_defaults(old_version)
        for section, options in defaults:
            for option in options:
                new_value = options[option]
                try:
                    old_value = old_defaults.get(section, option)
                except (cp.NoSectionError, cp.NoOptionError):
                    old_value = None
                if (old_value is None or to_text_string(new_value) != old_value):
                    self._set(section, option, new_value, verbose)

    def __remove_deprecated_options(self, old_version):
        """Remove options present in the .ini file but not in defaults."""
        old_defaults = self.__load_old_defaults(old_version)
        for section in old_defaults.sections():
            for option, _ in old_defaults.items(section, raw=self.raw):
                if self.get_default(section, option) is NoDefault:
                    self.remove_option(section, option)
                    if len(self.items(section, raw=self.raw)) == 0:
                        self.remove_section(section)

    def cleanup(self):
        """Remove .ini file associated to config."""
        os.remove(self.filename())

    def set_as_defaults(self):
        """Set defaults from the current config."""
        self.defaults = []
        for section in self.sections():
            secdict = {}
            for option, value in self.items(section, raw=self.raw):
                secdict[option] = value
            self.defaults.append((section, secdict))

    def get_defaults(self):
        self.reset_to_defaults(save=False)

        with io.StringIO() as fd:
            self.write(fd)
            fd.seek(0)
            return fd.read()

    def reset_to_defaults(self, save=True, verbose=False, section=None):
        """Reset config to Default values."""
        for sec, options in self.defaults:
            if section is None or section == sec:
                for option in options:
                    value = options[option]
                    self._set(sec, option, value, verbose)
        if save:
            self._save()

    def __check_section_option(self, section, option):
        """Private method to check section and option types."""
        if section is None:
            section = self.DEFAULT_SECTION_NAME
        elif not is_text_string(section):
            raise RuntimeError("Argument 'section' must be a string")
        if not is_text_string(option):
            raise RuntimeError("Argument 'option' must be a string")
        return section

    def get_default(self, section, option):
        """
        Get Default value for a given (section, option).

        -> useful for type checking in 'get' method
        """
        section = self.__check_section_option(section, option)
        for sec, options in self.defaults:
            if sec == section and option in options:
                return options[option]
        else:
            return NoDefault

    def get(self, section, option, default=NoDefault):
        """
        Get an option.

        section=None: attribute a default section name
        default: default value (if not specified, an exception
        will be raised if option doesn't exist)
        """
        section = self.__check_section_option(section, option)

        if not self.has_section(section):
            if default is NoDefault:
                raise cp.NoSectionError(section)
            else:
                self.add_section(section)

        if not self.has_option(section, option):
            if default is NoDefault:
                raise cp.NoOptionError(option, section)
            else:
                self.set(section, option, default)
                return default

        value = cp.ConfigParser.get(self, section, option, raw=self.raw)
        default_value = self.get_default(section, option)

        if isinstance(default_value, bool):
            value = ast.literal_eval(value)
        elif isinstance(default_value, float):
            value = float(value)
        elif isinstance(default_value, int):
            value = int(value)
        else:
            if PY2 and is_text_string(default_value):
                try:
                    value = value.decode('utf-8')
                except (UnicodeEncodeError, UnicodeDecodeError):
                    pass
            try:
                # lists, tuples, ...
                value = ast.literal_eval(value)
            except Exception:
                pass
        return value

    def set_default(self, section, option, default_value):
        """
        Set Default value for a given (section, option).

        -> called when a new (section, option) is set and no default exists
        """
        section = self.__check_section_option(section, option)
        for sec, options in self.defaults:
            if sec == section:
                options[option] = default_value

    def set(self, section, option, value, verbose=False, save=True):
        """
        Set an option.

        section=None: attribute a default section name
        """
        section = self.__check_section_option(section, option)
        default_value = self.get_default(section, option)
        if default_value is NoDefault:
            # This let us save correctly string value options with
            # no config default that contain non-ascii chars in
            # Python 2
            if PY2 and is_text_string(value):
                value = repr(value)
            default_value = value
            self.set_default(section, option, default_value)

        if isinstance(default_value, bool):
            value = bool(value)
        elif isinstance(default_value, float):
            value = float(value)
        elif isinstance(default_value, int):
            value = int(value)
        elif not is_text_string(default_value):
            value = repr(value)
        self._set(section, option, value, verbose)
        if save:
            self._save()

    def remove_section(self, section):
        """Remove the specified section from the configuration."""
        cp.ConfigParser.remove_section(self, section)
        self._save()

    def remove_option(self, section, option):
        """Remove the specified option from the specified section."""
        cp.ConfigParser.remove_option(self, section, option)
        self._save()
